import pandas as pd
from pymongo import MongoClient
from flask import Flask,request,jsonify
import time
import logging
import traceback
import numpy as np
from sklearn.preprocessing import StandardScaler
from sklearn.cluster import DBSCAN
import numbers
from bson.decimal128 import Decimal128
from common.database_dml import get_data_from_mongo,insert_data_into_mongo
app = Flask('processing_limit_power_by_statistics_wind——service')


@app.route('/hello', methods=['POST'])
def hello():
    return jsonify(message='Hello, World!')


def wind_statistics_judgement(df_power,args):
    """
    原理：基于实测辐照度与实际功率相关性强正相关，呈严格线性关系为假设前提，
      假设误差大致呈现标准正态分布 mean + N*std   
    """

    col_ws, col_power, eps, min_samples, ws_in, ws_out, ws_rated, cap = (args['col_ws'], args['col_power'], float(args['eps']),
                                    int(args['min_samples']), float(args['ws_in']), float(args['ws_out']), float(args['ws_rated']), float(args['cap']))
    print("min_sample", min_samples)

    df_power[col_ws] = df_power[col_ws].apply(
        lambda x: float(x.to_decimal()) if isinstance(x, Decimal128) else float(x) if isinstance(x,
                                                                                                 numbers.Number) else np.nan)
    df_power[col_power] = df_power[col_power].apply(
        lambda x: float(x.to_decimal()) if isinstance(x, Decimal128) else float(x) if isinstance(x,
                                                                                                 numbers.Number) else np.nan)
    df_tmp = df_power[(~np.isnan(df_power[col_ws])) & (~np.isnan(df_power[col_power]))]
    # 标准化数据
    data = df_tmp[[col_ws, col_power]].values
    scaler = StandardScaler()
    data_scaled = scaler.fit_transform(data)

    # DBSCAN 参数：eps是邻域半径，min_samples是构成核心点的最小样本数
    dbscan = DBSCAN(eps=eps, min_samples=min_samples)
    labels = dbscan.fit_predict(data_scaled)

    # 将结果添加到数据框
    df_tmp['is_limit'] = labels
    df_tmp['is_limit'] = df_tmp['is_limit'].apply(lambda x: True if x == -1 else False)

    df_tmp.loc[(df_tmp[col_ws] <= 0) & (df_tmp[col_power] > 0), 'is_limit'] = True
    # 切入、切出风速
    if ws_in != -99 and ws_out != -99:
        df_tmp.loc[(df_tmp[col_ws] >= ws_in * (1 + 0.5)) & (df_tmp[col_ws] < ws_out) & (
                    df_tmp[col_power] == 0), 'is_limit'] = True
    # 额定风速 是风机在正常工作并以最佳效率运行时所需的最小风速
    if ws_rated != -99:
        df_tmp.loc[(df_tmp[col_ws] >= ws_rated) & (df_tmp[col_power] >= cap(1 - 0.03)), 'is_limit'] = False

    df_tmp['c'] = df_tmp['is_limit'].apply(lambda x: 'red' if x == True else 'green')
    df_tmp.plot.scatter(x=col_ws, y=col_power, c='c')
    print(f"原始样本数：{df_power.shape[0]},异常点样本数：{df_tmp[df_tmp['is_limit'] == True].shape[0]},剩余样本数占比："
          f"{df_tmp[df_tmp['is_limit'] == False].shape[0] / df_power.shape[0]}")
    # return df_tmp[df_tmp['is_limit'] == False].drop(['is_limit','c'],axis=1)
    return df_tmp.drop(['c'],axis=1)


@app.route('/processing_limit_power_by_statistics_wind', methods=['POST'])
def processing_limit_power_by_statistics_wind():
    # 获取程序开始时间  
    start_time = time.time()  
    result = {}
    success = 0
    print("Program starts execution!")
    try:
        args = request.values.to_dict()
        print('args',args)
        logger.info(args)
        power_df = get_data_from_mongo(args)
        res_df = wind_statistics_judgement(power_df,args)
        insert_data_into_mongo(res_df,args)
        success = 1
    except Exception as e:
        my_exception = traceback.format_exc()
        my_exception.replace("\n","\t")
        result['msg'] = my_exception
    end_time = time.time() 
   
    result['success'] = success
    result['args'] = args
    result['start_time'] = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(start_time))
    result['end_time'] = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(end_time))
    print("Program execution ends!")
    return result

    
if __name__=="__main__":
    print("Program starts execution!")
    logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    logger = logging.getLogger("统计法清洗风电场站限电")
    from waitress import serve
    serve(app, host="0.0.0.0", port=10093)
    print("server start!")
    
   
    
